export interface Route<RouteName, RouteParams, RouteMeta> {
    name: RouteName;
    path: ((params: RouteParams) => string) | string;
    meta: RouteMeta;
}
export interface LandingProps {
    redirected?: boolean;
    unloading?: boolean;
    replaced?: boolean;
}
export declare type RouteLocation<LocationRoute, LocationMeta> = LocationRoute extends Route<infer RouteName, infer RouteParams, infer _RouteMeta> ? {
    name: RouteName;
    params: RouteParams;
    query: RouteQuery;
    hash: string;
    meta: LocationMeta;
    landing: LandingProps;
} : never;
export declare type RouteNotFoundLocation<NotFoundMeta> = RouteLocation<Route<'404', undefined, NotFoundMeta>, NotFoundMeta>;
export interface RouteQuery {
    [key: string]: string | boolean | number;
}
export interface RouteParams {
    [key: string]: string;
}
export declare type RouteRef<LinkRoute> = LinkRoute extends Route<infer RouteName, infer RouteParams, infer _RouteMeta> ? {
    name: RouteName;
    query?: RouteQuery;
    hash?: string;
} & (RouteParams extends undefined ? {
    params?: undefined;
} : {
    params: RouteParams;
}) : never;
export declare type InferRoute<Routes> = Routes extends (infer InferredRoute)[] ? InferredRoute : never;
export declare type InferRouteName<Routes> = Routes extends (infer InferredRoute)[] ? InferredRoute extends Route<any, any, any> ? InferredRoute['name'] : never : never;
export declare type InferRouteRef<Routes> = RouteRef<InferRoute<Routes>>;
export declare const notFoundLocation: RouteNotFoundLocation<{}>;
export declare function route<RouteName extends string, Path extends ((params: any) => string) | string, RouteMeta = {}>(name: RouteName, path: Path, meta?: RouteMeta): Path extends (() => string) | string ? Route<RouteName, undefined, RouteMeta> : Path extends (params: infer RouteParams) => string ? Route<RouteName, RouteParams, RouteMeta> : never;
export declare function createRouterCore<AppRoutes extends Array<Route<any, any, any>>>(appRoutes: AppRoutes): {
    resolveLocation: (url: string) => RouteLocation<InferRoute<AppRoutes>, InferRoute<AppRoutes>["meta"]> | {
        name: "404";
        params: undefined;
        query: RouteQuery;
        hash: string;
        meta: InferRoute<AppRoutes>["meta"];
        landing: LandingProps;
    };
    refToLocation: (ref: RouteRef<InferRoute<AppRoutes>>, landing?: LandingProps) => RouteLocation<InferRoute<AppRoutes>, InferRoute<AppRoutes>["meta"]> | {
        name: "404";
        params: undefined;
        query: RouteQuery;
        hash: string;
        meta: InferRoute<AppRoutes>["meta"];
        landing: LandingProps;
    };
    buildHref: (ref: RouteRef<InferRoute<AppRoutes>>) => string;
};
