"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouterCore = exports.route = exports.notFoundLocation = void 0;
exports.notFoundLocation = {
    name: '404',
    params: undefined,
    query: {},
    hash: '',
    meta: {},
    landing: {}
};
function route(name, path, meta) {
    // @ts-ignore: I've not idea how to make it happy ¯\_(ツ)_/¯
    return { name, path, meta: meta || {} };
}
exports.route = route;
function createRouterCore(appRoutes) {
    function resolveLocation(url) {
        const { pathname, searchParams, hash: unprocessedHash } = new URL(url);
        const query = searchParamsToQuery(searchParams);
        // Remove leading #
        const hash = unprocessedHash.slice(1);
        for (let index = 0; index < appRoutes.length; index++) {
            const route = appRoutes[index];
            const { name, path, meta } = route;
            const regExp = pathToRegExp(pathToMatchString(path));
            const captures = pathname.match(regExp);
            if (captures) {
                const params = parseParams(pathToMatchString(path), captures.slice(1)) || {};
                // @ts-ignore: No idea what's wrong
                return {
                    // @ts-ignore
                    name,
                    query,
                    params,
                    hash,
                    meta,
                    landing: {}
                };
            }
        }
        return {
            name: '404',
            params: undefined,
            query,
            hash,
            meta: {},
            landing: {}
        };
    }
    function refToLocation(ref, landing = {}) {
        const { name, params, query, hash = '' } = ref;
        const route = appRoutes.find(route => route.name === name);
        if (route) {
            const { meta } = route;
            return {
                name,
                query: query || {},
                params,
                hash: hash || '',
                meta,
                landing
            };
        }
        else {
            return exports.notFoundLocation;
        }
    }
    function buildHref(ref) {
        const { name } = ref;
        const route = appRoutes.find(route => route.name === name);
        if (!route) {
            throw new Error(`Can't find ${name} route`);
        }
        const { params, query, hash } = ref;
        const { path } = route;
        let href = params
            ? pathToMatchString(path).replace(/:([^/]+)/g, (_, param) => {
                if (!(param in params)) {
                    throw new Error(`Can't generate the path for ${name} route: ${param} param is missing in the passed params (${JSON.stringify(params)})`);
                }
                return params[param].toString();
            })
            : typeof path === 'string'
                ? path
                : path(undefined);
        const notEmpty = query && Object.keys(query).length > 0;
        if (query && notEmpty) {
            const searchParams = new URLSearchParams();
            Object.keys(query).forEach(key => {
                searchParams.set(key, query[key].toString());
            });
            href += `?${searchParams.toString()}`;
        }
        if (hash)
            href += `#${hash}`;
        return href;
    }
    return { resolveLocation, refToLocation, buildHref };
}
exports.createRouterCore = createRouterCore;
function pathToRegExp(path) {
    return new RegExp(`^${path.replace(/(:[^/]+)/g, '([^/]*)')}/?$`);
}
function parseParams(path, values) {
    const segmentNameCaptures = path.match(/:([^/]+)/g);
    if (segmentNameCaptures) {
        const segmentNames = segmentNameCaptures.map(str => str.slice(1));
        const segments = {};
        segmentNames.forEach((name, index) => {
            segments[name] = values[index];
        });
        return segments;
    }
}
function searchParamsToQuery(params) {
    const query = {};
    params.forEach((value, key) => {
        query[key] = parseSearchParamValue(value);
    });
    return query;
}
function parseSearchParamValue(value) {
    if (value === 'true') {
        return true;
    }
    else if (value === 'false') {
        return false;
    }
    else if (parseFloat(value).toString() === value) {
        return parseFloat(value);
    }
    else {
        return value;
    }
}
const paramsProxy = new Proxy({}, { get: (_, prop) => `:${prop.toString()}` });
function pathToMatchString(path) {
    return typeof path === 'string' ? path : path(paramsProxy);
}
//# sourceMappingURL=index.js.map