"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouter = void 0;
const adaptor = __importStar(require("./adaptor"));
const core = __importStar(require("./core"));
function createRouter(appRoutes, options = {}) {
    // Default options
    options.scrollOnMissingHash = options.scrollOnMissingHash || 'top';
    const { resolveLocation, refToLocation, buildHref } = core.createRouterCore(appRoutes);
    const RouterContext = adaptor.createContext({
        location: core.notFoundLocation,
        navigate: () => void 0,
        redirect: () => void 0,
        buildHref: () => ''
    });
    // Initial values
    let historyIndex = 0;
    function useRouter(initialURL) {
        const initialLocation = resolveLocation(initialURL);
        const [location, setLocation] = adaptor.useState(initialLocation);
        adaptor.useEffect(() => {
            // Are we in the browser environment?
            if (typeof window !== 'undefined') {
                let negateEvent = false;
                const popStateListener = (e) => {
                    var _a;
                    if (negateEvent) {
                        negateEvent = false;
                        return;
                    }
                    const popIndex = ((_a = e.state) === null || _a === void 0 ? void 0 : _a.index) === 'number' ? e.state.index : 0;
                    const delta = popIndex < historyIndex ? -1 : 1;
                    historyIndex = popIndex;
                    confirmUnload(true, () => {
                        const newLocation = resolveLocation(window.location.href);
                        setLocation(newLocation);
                    }, () => {
                        negateEvent = true;
                        history.go(-delta);
                    });
                };
                window.addEventListener('popstate', popStateListener);
                return () => window.removeEventListener('popstate', popStateListener);
            }
        }, []);
        // After location change scroll to the element with id equal hash
        // or the top of the page
        const prevLocation = adaptor.useRef(initialLocation);
        adaptor.useEffect(() => {
            // Skip on initial run
            if (location !== initialLocation) {
                const scrollToEl = (location.hash && document.getElementById(location.hash)) || undefined;
                if (scrollToEl) {
                    // If there's hash and element with id equal hash is found
                    // then scroll to it.
                    window.scroll(0, elementTop(scrollToEl));
                }
                else if (
                // If it's not the same page..
                !isSamePage(location, prevLocation.current) ||
                    // ...or hash changed but the element is not found
                    (location.hash !== prevLocation.current.hash &&
                        options.scrollOnMissingHash === 'top')) {
                    window.scroll(0, 0);
                }
            }
            prevLocation.current = location;
        }, [location]);
        function navigate(ref, { unloading = true, redirected, replaced } = {}) {
            const landing = { unloading, redirected };
            const currentLocation = resolveLocation(window.location.href);
            const newLocation = refToLocation(ref, landing);
            confirmUnload(!unloading, () => {
                if (isSamePage(currentLocation, ref)) {
                    // Update hash if changed
                    if (currentLocation.hash !== ref.hash)
                        window.location.hash = ref.hash || '';
                    // If the hash is present and the object with given id is found
                    // then scroll to it
                    const scrollToEl = ref.hash && document.getElementById(ref.hash);
                    if (scrollToEl)
                        window.scroll(0, elementTop(scrollToEl));
                }
                else if (replaced) {
                    // Replace the state in the history
                    window.history.replaceState({ index: historyIndex }, '', buildHref(ref));
                    setLocation(newLocation);
                }
                else {
                    // Push the state to the history
                    historyIndex++;
                    window.history.pushState({ index: historyIndex }, '', buildHref(ref));
                    setLocation(newLocation);
                }
            });
        }
        function redirect(ref, landing) {
            navigate(ref, Object.assign(Object.assign({ unloading: false }, landing), { redirected: true }));
        }
        return { location, navigate, redirect, buildHref };
    }
    function RouterLink(_a) {
        var { to, target, rel, mode = 'anchor', tag, children, component, onClick: customOnClick } = _a, componentProps = __rest(_a, ["to", "target", "rel", "mode", "tag", "children", "component", "onClick"]);
        const { navigate } = adaptor.useContext(RouterContext);
        const href = buildHref(to);
        const componentTag = tag || mode === 'anchor' ? 'a' : 'div';
        const onClick = (e) => {
            // Handle the custom click handler
            customOnClick && customOnClick(e);
            // If default behavior got prevented by the custom click handler or
            // the mode is "anchor" (default) and user clicks the link with
            // the command or control key then use default browser behavior.
            if (e.defaultPrevented || (mode !== 'block' && (e.metaKey || e.ctrlKey)))
                return;
            // Otherwise, prevent the default behavior and stop propogation.
            e.preventDefault();
            e.stopPropagation();
            // When the target attribute is "_blank" or the mode is "block" and
            // user clicks the link with the command or control key
            // then open the link in a new window.
            if (componentProps.target === '_blank' ||
                (mode === 'block' && (e.metaKey || e.ctrlKey))) {
                // Preserve noreferrer and noopener
                const features = [];
                if (typeof componentProps.rel === 'string') {
                    if (componentProps.rel.includes('noreferrer'))
                        features.push('noreferrer');
                    if (componentProps.rel.includes('noopener'))
                        features.push('noopener');
                }
                window.open(href, '_blank', features.join(','));
            }
            else {
                navigate(to);
            }
        };
        const props = mode === 'anchor'
            ? Object.assign({ onClick, href, target, rel }, componentProps)
            : Object.assign({ onClick }, componentProps);
        if (component) {
            return adaptor.cloneElement(component, props, children);
        }
        else {
            return adaptor.createElement(componentTag, props, children);
        }
    }
    function isSamePage(locationA, locationB) {
        return (locationA.name === locationB.name &&
            isEqual(locationA.params, locationB.params) &&
            isEqual(locationA.query, locationB.query));
    }
    return {
        useRouter,
        RouterContext,
        RouterLink,
        resolveLocation,
        refToLocation,
        buildHref
    };
}
exports.createRouter = createRouter;
function elementTop(element) {
    return window.scrollY + element.getBoundingClientRect().y;
}
function isEqual(objA, objB) {
    if (typeof objA === 'object' && objA !== null) {
        const objAKeys = Object.keys(objA);
        const objBKeys = typeof objB === 'object' && objB !== null && Object.keys(objB);
        if (!objBKeys || objAKeys.length !== objBKeys.length)
            return false;
        return objAKeys.every(key => isEqual(objA[key], objB[key]));
    }
    else {
        return objA === objB;
    }
}
function confirmUnload(force, confirmCallback, cancelCallback) {
    if (force) {
        confirmCallback();
    }
    else {
        class BeforeUnloadEventPhony extends Event {
            constructor() {
                super('beforeunload');
            }
            get returnValue() {
                return this._returnValue;
            }
            set returnValue(value) {
                this._returnValue = value;
            }
        }
        const unloadEvent = new BeforeUnloadEventPhony();
        const listener = (e) => {
            if (!e.returnValue || confirm(e.returnValue))
                confirmCallback();
            else
                cancelCallback === null || cancelCallback === void 0 ? void 0 : cancelCallback();
            window.removeEventListener('beforeunload', listener);
        };
        window.addEventListener('beforeunload', listener);
        window.dispatchEvent(unloadEvent);
    }
}
//# sourceMappingURL=framework.js.map